using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Input.Touch;
using Microsoft.Xna.Framework.Media;

namespace DrawUserIndexedPrimitives
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        VertexPositionColor[] userPrimitives;
        short[] userPrimitivesIndices;
        BasicEffect basicEffect;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // Create the verticies for our triangle
            userPrimitives = new VertexPositionColor[4];
            userPrimitives[0] = new VertexPositionColor();
            userPrimitives[0].Position = new Vector3(-1, 1, 0);
            userPrimitives[0].Color = Color.Red;
            userPrimitives[1] = new VertexPositionColor();
            userPrimitives[1].Position = new Vector3(1, 1, 0);
            userPrimitives[1].Color = Color.Green;
            userPrimitives[2] = new VertexPositionColor();
            userPrimitives[2].Position = new Vector3(-1, -1, 0);
            userPrimitives[2].Color = Color.Blue;
            userPrimitives[3] = new VertexPositionColor();
            userPrimitives[3].Position = new Vector3(1, -1, 0);
            userPrimitives[3].Color = Color.Purple;

            // Create the indices used for each triangle
            userPrimitivesIndices = new short[6];
            // First triangle
            userPrimitivesIndices[0] = 0;
            userPrimitivesIndices[1] = 1;
            userPrimitivesIndices[2] = 2;
            // Second Triangle
            userPrimitivesIndices[3] = 3;

            // Create new basic effect and properites
            basicEffect = new BasicEffect(GraphicsDevice);
            basicEffect.World = Matrix.Identity;
            basicEffect.View = Matrix.CreateLookAt(new Vector3(0, 0, 3), new Vector3(0, 0, 0), new Vector3(0, 1, 0));
            basicEffect.Projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.PiOver4, GraphicsDevice.Viewport.AspectRatio, 0.1f, 100.0f);
            basicEffect.VertexColorEnabled = true;
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: Add your update logic here

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            // Start using the BasicEffect
            basicEffect.CurrentTechnique.Passes[0].Apply();
            // Draw the primitives
            GraphicsDevice.DrawUserIndexedPrimitives<VertexPositionColor>(PrimitiveType.TriangleStrip, userPrimitives, 0, 4, userPrimitivesIndices, 0, 2);

            base.Draw(gameTime);
        }
    }
}
