using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Input.Touch;
using Microsoft.Xna.Framework.Media;

namespace BasicEffectSample
{
    public enum BasicEffectModes
    {
        None,
        AmbientLights,
        OneLight,
        TwoLights,
        ThreeLights,
        DefaultLighting,
        Specular,
        Emissive,
        Textured,
        VertexColored,
        Fog,
    }
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        Model model;
        BasicEffect effect;
        BasicEffectModes currentMode = BasicEffectModes.Fog;
        Texture2D texture;
        VertexBuffer vb;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // TODO: use this.Content to load your game content here
            model = Content.Load<Model>("box");
            if ( (currentMode == BasicEffectModes.Specular)  ||
                (currentMode == BasicEffectModes.Emissive) ||
                (currentMode == BasicEffectModes.Fog))
            {
                model = Content.Load<Model>("sphere");
            }

            effect = new BasicEffect(GraphicsDevice);
            effect.Projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.PiOver4,
                GraphicsDevice.Viewport.AspectRatio, 1.0f, 100.0f);
            effect.View = Matrix.CreateLookAt(new Vector3(2, 3, 4), Vector3.Zero, Vector3.Up);
            switch (currentMode)
            {
                case BasicEffectModes.DefaultLighting:
                    effect.EnableDefaultLighting();
                    break;
                case BasicEffectModes.OneLight:
                    effect.LightingEnabled = true;
                    effect.DirectionalLight0.Enabled = true;
                    effect.DirectionalLight0.DiffuseColor = Color.Yellow.ToVector3();
                    effect.DirectionalLight0.Direction = Vector3.Normalize(new Vector3(-1,-1.5f,0));
                    break;
                case BasicEffectModes.TwoLights:
                    effect.LightingEnabled = true;
                    effect.DirectionalLight0.Enabled = true;
                    effect.DirectionalLight0.DiffuseColor = Color.Yellow.ToVector3();
                    effect.DirectionalLight0.Direction = Vector3.Normalize(new Vector3(-1,-1.5f,0));
                    effect.DirectionalLight1.Enabled = true;
                    effect.DirectionalLight1.DiffuseColor = Color.Blue.ToVector3();
                    effect.DirectionalLight1.Direction = Vector3.Left;
                    break;
                case BasicEffectModes.ThreeLights:
                    effect.LightingEnabled = true;
                    effect.DirectionalLight0.Enabled = true;
                    effect.DirectionalLight0.DiffuseColor = Color.Red.ToVector3();
                    effect.DirectionalLight0.Direction = Vector3.Normalize(new Vector3(-1,-1.5f,0));
                    effect.DirectionalLight1.Enabled = true;
                    effect.DirectionalLight1.DiffuseColor = Color.Blue.ToVector3();
                    effect.DirectionalLight1.Direction = Vector3.Normalize(new Vector3(1, -1.5f, -1));
                    effect.DirectionalLight2.Enabled = true;
                    effect.DirectionalLight2.DiffuseColor = Color.Green.ToVector3();
                    effect.DirectionalLight2.Direction = Vector3.Normalize(new Vector3(0, -1.5f, -1));
                    break;
                case BasicEffectModes.AmbientLights:
                    effect.LightingEnabled = true;
                    effect.DirectionalLight0.Enabled = false;
                    effect.AmbientLightColor = Color.Red.ToVector3();
                    effect.EmissiveColor = Color.Green.ToVector3();
                    break;
                case BasicEffectModes.Specular:
                    effect.LightingEnabled = true;
                    effect.DirectionalLight0.Enabled = true;
                    effect.DirectionalLight0.DiffuseColor = Color.Red.ToVector3();
                    effect.DirectionalLight0.Direction = Vector3.Normalize(new Vector3(0, -1.5f, -1));
                    effect.DirectionalLight0.SpecularColor = Color.White.ToVector3();
                    effect.SpecularColor = Color.White.ToVector3();
                    effect.SpecularPower = 16.0f;
                    effect.PreferPerPixelLighting = true;
                    break;
                case BasicEffectModes.Emissive:
                    effect.LightingEnabled = true;
                    effect.DirectionalLight0.Enabled = true;
                    effect.DirectionalLight0.DiffuseColor = Color.Blue.ToVector3();
                    effect.DirectionalLight0.Direction = Vector3.Normalize(new Vector3(-1, -1.5f, 0));
                    effect.EmissiveColor = Color.Red.ToVector3();
                    break;
                case BasicEffectModes.Textured:
                    texture = Content.Load<Texture2D>("cat");
                    effect.TextureEnabled = true;
                    effect.Texture = texture;
                    effect.EnableDefaultLighting();
                    break;
                case BasicEffectModes.VertexColored:
                    effect.VertexColorEnabled = true;
                    int numberVertices = model.Meshes[0].MeshParts[0].NumVertices;
                    vb = new VertexBuffer(GraphicsDevice, typeof(VertexColor), 
                        numberVertices, BufferUsage.None);
                    VertexPositionNormalTexture[] data = new VertexPositionNormalTexture[numberVertices];
                    VertexColor[] newData = new VertexColor[numberVertices];
                    model.Meshes[0].MeshParts[0].VertexBuffer.GetData<VertexPositionNormalTexture>(
                        data);
                    // Copy the data
                    for (int i = 0; i < numberVertices; i++)
                    {
                        newData[i] = new VertexColor(data[i]);
                        newData[i].Color = Color.Red;
                    }
                    vb.SetData<VertexColor>(newData);
                    texture = Content.Load<Texture2D>("cat");
                    effect.Texture = texture;
                    effect.TextureEnabled = true;
                    effect.EnableDefaultLighting();
                    break;
                case BasicEffectModes.Fog:
                    effect.View = Matrix.CreateLookAt(new Vector3(0, 8, 22), Vector3.Zero, Vector3.Up);
                    effect.EnableDefaultLighting();
                    effect.SpecularColor = Vector3.Zero;
                    effect.PreferPerPixelLighting = true;
                    effect.FogColor = Color.CornflowerBlue.ToVector3();
                    effect.FogEnabled = true;
                    effect.FogStart = 10.0f;
                    effect.FogEnd = 40.0f;
                    break;
            }
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: Add your update logic here
            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);
            switch(currentMode)
            {
                case BasicEffectModes.None:
                case BasicEffectModes.AmbientLights:
                case BasicEffectModes.OneLight:
                case BasicEffectModes.TwoLights:
                case BasicEffectModes.ThreeLights:
                case BasicEffectModes.DefaultLighting:
                case BasicEffectModes.Specular:
                case BasicEffectModes.Emissive:
                case BasicEffectModes.Textured:
                    DrawModel(model, Matrix.Identity, effect);
                    break;
                case BasicEffectModes.VertexColored:
                    DrawModelWithVertexBuffer(model, vb, Matrix.Identity, effect);
                break;
                case BasicEffectModes.Fog:
                    const int NumberSpheres = 55;
                    for (int i = 0; i < NumberSpheres; i++)
                    {
                        Matrix world = Matrix.CreateTranslation(
                            ((i % 5) - 2) * 4, 0, (i/5-3) * -4);
                        DrawModel(model, world, effect);
                    }
                break;
            }
        }

        private Matrix GetParentTransform(Model m, ModelBone mb)
        {
            return (mb == m.Root) ? mb.Transform :
                mb.Transform * GetParentTransform(m, mb.Parent);
        }
        private void DrawModel(Model m, Matrix world, BasicEffect be)
        {
            foreach (ModelMesh mm in model.Meshes)
            {
                foreach (ModelMeshPart mmp in mm.MeshParts)
                {
                    be.World = GetParentTransform(m, mm.ParentBone) * world;
                    GraphicsDevice.SetVertexBuffer(mmp.VertexBuffer, mmp.VertexOffset);
                    GraphicsDevice.Indices = mmp.IndexBuffer;
                    be.CurrentTechnique.Passes[0].Apply();
                    GraphicsDevice.DrawIndexedPrimitives(
                        PrimitiveType.TriangleList, 0, 0,
                        mmp.NumVertices, mmp.StartIndex, mmp.PrimitiveCount);
                }
            }
        }
        private void DrawModelWithVertexBuffer(Model m, VertexBuffer v, 
            Matrix world, BasicEffect be)
        {
            foreach (ModelMesh mm in model.Meshes)
            {
                foreach (ModelMeshPart mmp in mm.MeshParts)
                {
                    be.World = GetParentTransform(m, mm.ParentBone) * world;
                    GraphicsDevice.SetVertexBuffer(v, 0);
                    GraphicsDevice.Indices = mmp.IndexBuffer;
                    be.CurrentTechnique.Passes[0].Apply();
                    GraphicsDevice.DrawIndexedPrimitives(
                        PrimitiveType.TriangleList, 0, 0,
                        mmp.NumVertices, mmp.StartIndex, mmp.PrimitiveCount);
                }
            }
        }

    }
    struct VertexColor : IVertexType
    {
        public Vector3 Position;
        public Vector3 Normal;
        public Vector2 UV;
        public Color Color;
        public static readonly VertexDeclaration _decl = new VertexDeclaration(
            new VertexElement(0, VertexElementFormat.Vector3, 
                VertexElementUsage.Position, 0),
            new VertexElement(12, VertexElementFormat.Vector3, 
                VertexElementUsage.Normal, 0),
            new VertexElement(24, VertexElementFormat.Vector2, 
                VertexElementUsage.TextureCoordinate, 0),
            new VertexElement(32, VertexElementFormat.Color, 
                VertexElementUsage.Color, 0));
        public VertexColor(VertexPositionNormalTexture data)
        {
            Position = data.Position;
            Normal = data.Normal;
            UV = data.TextureCoordinate;
            Color = Color.White;
        }
        public VertexDeclaration VertexDeclaration
        {
            get { return _decl; }
        }
    }
}
