using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace DepthSample
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        Model model;
        Matrix proj;
        Matrix view;
        RasterizerState wireframe;
        RasterizerState scissor;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // TODO: use this.Content to load your game content here
            proj = Matrix.CreatePerspectiveFieldOfView(MathHelper.PiOver4,
                GraphicsDevice.Viewport.AspectRatio, 1.0f, 100.0f);
            view = Matrix.CreateLookAt(new Vector3(0, 3, 20), Vector3.Zero, Vector3.Up);

            model = Content.Load<Model>("depthmodel");
            foreach (ModelMesh mm in model.Meshes)
            {
                foreach (Effect e in mm.Effects)
                {
                    IEffectLights iel = e as IEffectLights;
                    if (iel != null)
                    {
                        iel.EnableDefaultLighting();
                    }
                }
            }

            wireframe = new RasterizerState();
            wireframe.FillMode = FillMode.WireFrame;

            scissor = new RasterizerState();
            scissor.ScissorTestEnable = true;
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: Add your update logic here

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);
            float time = (float)gameTime.TotalGameTime.TotalSeconds;
            Matrix rotation = Matrix.CreateRotationZ(time) * 
                Matrix.CreateRotationY(time / 4.0f);
            Matrix world = Matrix.CreateScale(0.25f) * rotation;

            // Set some rasterizer states
            GraphicsDevice.RasterizerState = RasterizerState.CullCounterClockwise;
            model.Draw(world * Matrix.CreateTranslation(-6, 0, 0), view, proj);
            GraphicsDevice.RasterizerState = wireframe;
            model.Draw(world * Matrix.CreateTranslation(6, 0, 0), view, proj);
            GraphicsDevice.RasterizerState = RasterizerState.CullClockwise;
            model.Draw(world * Matrix.CreateTranslation(0, -6, 0), view, proj);
            GraphicsDevice.RasterizerState = scissor;
            GraphicsDevice.ScissorRectangle = new Rectangle(0, 0, 
                GraphicsDevice.Viewport.Width, GraphicsDevice.Viewport.Height / 8);
            model.Draw(world * Matrix.CreateTranslation(0, 6, 0), view, proj);
        }
    }
}
