using System;
using System.IO;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content.Pipeline;
using Microsoft.Xna.Framework.Content.Pipeline.Graphics;
using Microsoft.Xna.Framework.Content.Pipeline.Processors;

namespace ContentPipelineExtension
{

    /// <summary>
    /// This class will be instantiated by the XNA Framework Content Pipeline
    /// to apply custom processing to content data, converting an object of
    /// type TInput to TOutput. The input and output types may be the same if
    /// the processor wishes to alter data without changing its type.
    ///
    /// This should be part of a Content Pipeline Extension Library project.
    ///
    /// TODO: change the ContentProcessor attribute to specify the correct
    /// display name for this processor.
    /// </summary>
    [ContentImporter(".mylevel", DisplayName="My Level Importer")]
    public class MyContentImporter : ContentImporter<ContentImporter.Level>
    {
        public override ContentImporter.Level Import(string filename, 
            ContentImporterContext context)
        {
            ContentImporter.Level level = new ContentImporter.Level();
            level.PositionList = new List<Vector3>();
            level.RotationList = new List<Vector3>();
            using (StreamReader reader = new StreamReader(File.OpenRead(filename)))
            {
                while (!reader.EndOfStream)
                {
                    string current = reader.ReadLine();
                    string[] floats = current.Split(' ');
                    level.PositionList.Add(new Vector3(float.Parse(floats[0]),
                        float.Parse(floats[1]), float.Parse(floats[2])));
                    level.RotationList.Add(new Vector3(float.Parse(floats[3]),
                        float.Parse(floats[4]), float.Parse(floats[5])));
                }
            }

            return level;
        }
    }
}