using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace AvatarAnimationBlending
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        AvatarDescription avatarDescription;
        AvatarAnimation[] avatarAnimations;
        AvatarRenderer avatarRenderer;

        Matrix world;
        Matrix view;
        Matrix projection;

        GamePadState lastGamePadState;

        AnimationBlender animationBlender;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            graphics.PreferredBackBufferWidth = 1280;
            graphics.PreferredBackBufferHeight = 720;
            Content.RootDirectory = "Content";

            Components.Add(new GamerServicesComponent(this));
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // Load random avatar
            avatarDescription = AvatarDescription.CreateRandom();
            // Load all of the animation
            avatarAnimations = new AvatarAnimation[3];
            avatarAnimations[0] = new AvatarAnimation(AvatarAnimationPreset.Celebrate);
            avatarAnimations[1] = new AvatarAnimation(AvatarAnimationPreset.Clap);
            avatarAnimations[2] = new AvatarAnimation(AvatarAnimationPreset.Wave);

            // Create the animation blender
            animationBlender = new AnimationBlender(avatarAnimations[0]);

            // Load avatar renderer for random description
            avatarRenderer = new AvatarRenderer(avatarDescription);

            // Rotate avatar to face the screen
            world = Matrix.CreateRotationY(MathHelper.Pi);
            // Set view and projection matrices to something sensible
            view = Matrix.CreateLookAt(new Vector3(0, 1, 3), new Vector3(0, 1, 0), Vector3.Up);
            projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.PiOver4,
                                                             GraphicsDevice.Viewport.AspectRatio,
                                                             0.01f, 200.0f);
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // Update the animation blender
            animationBlender.Update(gameTime.ElapsedGameTime, true);

            // Get the current state of player one controller
            GamePadState currentGamePadState = GamePad.GetState(PlayerIndex.One);

            // Create new random avatar if the user presses A
            if (currentGamePadState.Buttons.A == ButtonState.Pressed &&
                lastGamePadState.Buttons.A == ButtonState.Released)
            {
                // Generate new random description
                avatarDescription = AvatarDescription.CreateRandom();
                avatarRenderer = new AvatarRenderer(avatarDescription);
            }

            // If the user presses the X, Y, or B buttons blend to an animation
            if (currentGamePadState.Buttons.X == ButtonState.Pressed &&
                lastGamePadState.Buttons.X == ButtonState.Released)
            {
                animationBlender.Play(avatarAnimations[0]);
            }
            if (currentGamePadState.Buttons.Y == ButtonState.Pressed &&
                lastGamePadState.Buttons.Y == ButtonState.Released)
            {
                animationBlender.Play(avatarAnimations[1]);
            }
            if (currentGamePadState.Buttons.B == ButtonState.Pressed &&
                lastGamePadState.Buttons.B == ButtonState.Released)
            {
                animationBlender.Play(avatarAnimations[2]);
            }

            // Save the state so we can prevent repeat button presses
            lastGamePadState = currentGamePadState;

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            // Set the world, view, and projection matrices
            avatarRenderer.World = world;
            avatarRenderer.View = view;
            avatarRenderer.Projection = projection;

            // Draw the avatar to the current render target
            avatarRenderer.Draw(animationBlender);

            base.Draw(gameTime);
        }
    }
}
