using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Input.Touch;
using Microsoft.Xna.Framework.Media;
using System.Device.Location; 


namespace LocationInput
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        SpriteFont spriteFont;

        GeoCoordinateWatcher geoWatcher;
        GeoCoordinate currentCoordinate;
        GeoPositionStatus geoStatus;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            // Frame rate is 30 fps by default for Windows Phone.
            TargetElapsedTime = TimeSpan.FromTicks(333333);

            // Pre-autoscale settings.
            graphics.PreferredBackBufferWidth = 480;
            graphics.PreferredBackBufferHeight = 800;
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            geoWatcher = new GeoCoordinateWatcher(GeoPositionAccuracy.High);
            geoWatcher.StatusChanged += new EventHandler<GeoPositionStatusChangedEventArgs>(geoWatcher_StatusChanged);
            geoWatcher.PositionChanged += new EventHandler<GeoPositionChangedEventArgs<GeoCoordinate>>(geoWatcher_PositionChanged);
            geoWatcher.MovementThreshold = 20;
            geoWatcher.Start();

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            spriteFont = Content.Load<SpriteFont>("SpriteFont1");
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: Add your update logic here

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            string locationInfo = "Status: " + geoStatus.ToString() + "\n";
            if (currentCoordinate != null)
            {
                locationInfo += "Latitude: " + currentCoordinate.Latitude.ToString("0.000") + "\n";
                locationInfo += "Longitude: " + currentCoordinate.Longitude.ToString("0.000") + "\n";
                locationInfo += "Altitude: " + currentCoordinate.Altitude.ToString("0.000") + "\n";
                locationInfo += "Course: " + currentCoordinate.Course.ToString("0.000") + "\n";
                locationInfo += "Speed: " + currentCoordinate.Speed.ToString("0.000") + "\n";
                locationInfo += "HorizontalAccuracy: " + currentCoordinate.HorizontalAccuracy.ToString("0.000") + "\n";
                locationInfo += "VerticalAccuracy: " + currentCoordinate.VerticalAccuracy.ToString("0.000") + "\n";
            }

            spriteBatch.Begin();
            spriteBatch.DrawString(spriteFont, locationInfo, new Vector2(10, 50), Color.White);
            spriteBatch.End();

            base.Draw(gameTime);
        }

        protected override void OnExiting(object sender, EventArgs args)
        {
            geoWatcher.Stop();

            base.OnExiting(sender, args);
        }

        void geoWatcher_StatusChanged(object sender, GeoPositionStatusChangedEventArgs e)
        {
            // Check the current status
            geoStatus = e.Status;
            switch (geoStatus)
            {
                case GeoPositionStatus.Initializing:
                    // The location service is still starting up
                    break;
                case GeoPositionStatus.Ready:
                    // The location service is running and returning location data
                    break;
                case GeoPositionStatus.NoData:
                    // The locaiton has started but can not current find its location
                    break;
                case GeoPositionStatus.Disabled:
                    if (geoWatcher.Permission == GeoPositionPermission.Denied)
                    {
                        // The user has turned off location services
                    }
                    else
                    {
                        // The location service failed to start
                    }
                    break;
            }

        }

        void geoWatcher_PositionChanged(object sender, GeoPositionChangedEventArgs<GeoCoordinate> e)
        {
            // Store the current coordinate
            currentCoordinate = e.Position.Location;
        }

    }
}
