using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Input.Touch;
using Microsoft.Xna.Framework.Media;

namespace MultitouchInput
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        Texture2D spriteTexture;
        Vector2 spritePosition;
        Vector2 spriteCenter;
        float rotation = 0;
        float scale = 1;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            // Frame rate is 30 fps by default for Windows Phone.
            TargetElapsedTime = TimeSpan.FromTicks(333333);

            // Pre-autoscale settings.
            graphics.PreferredBackBufferWidth = 480;
            graphics.PreferredBackBufferHeight = 800;

            spritePosition = new Vector2(240, 400);
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // Query the capabilities of the device
            TouchPanelCapabilities touchCaps = TouchPanel.GetCapabilities();
            // Check to see if a device is connected
            if (touchCaps.IsConnected)
            {
                // The max number of TouchLocation points that can be tracked
                int maxPoints = touchCaps.MaximumTouchCount;
            }

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            spriteTexture = Content.Load<Texture2D>("XnaLogo");
            spriteCenter.X = spriteTexture.Width / 2.0f;
            spriteCenter.Y = spriteTexture.Height / 2.0f;
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // Get current touch state
            TouchCollection currentTouchState = TouchPanel.GetState();

            // Loop over each touch point
            for (int i = 0; i < currentTouchState.Count; i++)
            {
                TouchLocation currentLocation = currentTouchState[i];
             
                // Only process points that were just pressed or are moving
                if (currentLocation.State == TouchLocationState.Pressed || 
                    currentLocation.State == TouchLocationState.Moved)
                {
                    // Use the 1st point for the sprite location
                    if (i == 0)
                    {
                        spritePosition = currentLocation.Position;
                    }
                    // Use the 2nd point for rotation and scale
                    else if (i == 1)
                    {
                        // Check distance between the 1st and 2nd points
                        Vector2 positionToSprite = currentLocation.Position - spritePosition;
                        float distance = positionToSprite.Length();
                        scale = distance / (spriteTexture.Width / 2);

                        // Normalize to get direction vector
                        positionToSprite.Normalize();
                        // Use the direction between fingers to determine rotation angle
                        float cosAngle = Vector2.Dot(positionToSprite, Vector2.UnitX);
                        if (spritePosition.Y > currentLocation.Position.Y)
                            rotation = 2 * MathHelper.Pi - (float)Math.Acos(cosAngle);
                        else
                            rotation = (float)Math.Acos(cosAngle);
                    }
                    else
                    {
                        break;
                    }
                }
            }

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            // Draw the sprite
            spriteBatch.Begin();
            spriteBatch.Draw(spriteTexture, spritePosition, null, Color.White,
                             rotation, spriteCenter, scale, SpriteEffects.None, 0);
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
