using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Input.Touch;
using Microsoft.Xna.Framework.Media;

namespace GuideSample
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        GamePadState lastState;
        SpriteFont font;
        SignedInGamer lastGamer;
        GamerProfile lastGamerProfile;
        Texture2D gamerPicture;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            SignedInGamer.SignedIn += new EventHandler<SignedInEventArgs>((o,e) =>
            {
                lastGamer = e.Gamer;

                lastGamer.BeginGetProfile(new AsyncCallback( (result) =>
                    {
                        lastGamerProfile =
                            lastGamer.EndGetProfile(result);

                        gamerPicture = Texture2D.FromStream(
                            GraphicsDevice, lastGamerProfile.GetGamerPicture());
                    }), null);
            });
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            Components.Add(new GamerServicesComponent(this));

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            font = Content.Load<SpriteFont>("font");
            // TODO: use this.Content to load your game content here
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: Add your update logic here
            GamePadState state = GamePad.GetState(PlayerIndex.One);

            if (WasButtonPressed(Buttons.A, state, lastState))
            {
                if (lastGamer != null)
                {
                    Guide.ShowGamerCard(PlayerIndex.One, lastGamer);
                }
            }

            lastState = state;
            base.Update(gameTime);
        }

        /// <summary>
        /// Simple helper to determine if a button was pressed
        /// </summary>
        public bool WasButtonPressed(Buttons button, GamePadState current, GamePadState last)
        {
            return (current.IsButtonUp(button) && last.IsButtonDown(button));
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            spriteBatch.Begin();
            if (gamerPicture != null)
            {
                spriteBatch.Draw(gamerPicture, new Vector2(400,100), Color.White);
            }
            if (lastGamer != null)
            {
                lastGamer.Presence.PresenceMode = GamerPresenceMode.OnARoll;
                Vector2 pos = new Vector2(50, 30);
                spriteBatch.DrawString(font, lastGamer.Gamertag, pos, Color.White);
                if (lastGamerProfile != null)
                {
                    pos.Y += 20;
                    spriteBatch.DrawString(font, lastGamerProfile.GamerScore.ToString(), 
                        pos, Color.White);
                }
            }
            spriteBatch.End();
        }

    }
}
